# sb::Admin::Category
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Admin::Category;
use strict;
use sb::Utils;
use sb::Admin::Entry ();
use base qw( sb::Admin::Entry );
use vars qw( $VERSION );
$VERSION = '0.06';
# 0.06 [2007/12/07] modified _update_edit_category to removed a using wrong variable.
# 0.05 [2007/11/05] added title_name
# 0.04 [2007/09/24] fixed a bug in _count_entry
# 0.03 [2007/09/20] ported from Serene Bach 2.*
# 0.02 [2007/02/06] changed @ISA, _display_category_list, and _update_category to handle new description
# 0.01 [2006/11/09] changed _display_category_list / changed _update_category to check index directory
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines
sub TEMPLATE       (){ 'category.html' };
sub JSFILE_NAME    (){ 'script/listtable.js' }
sub LIST_COLUMN    (){ 8 };
sub DEFAULT_COLUMN (){ '-' };
sub ORDER_LEFT     (){ '<input type="submit" name="up%d" value="&#9650;" class="updown" />' };
sub ORDER_RIGHT    (){ '<input type="submit" name="dn%d" value="&#9660;" class="updown" />' };
sub ORDER_COLUMN   (){ '%s</td><td>%s' };
sub NAVI_ARROW     (){ ' &gt; ' };
# --------------------------------------
# // override base class
sub js_paths
{
	my $self = shift;
	my $editor = $self->get_editor();
	return( $self->static_path . JSFILE_NAME, $editor->javascripts() );
}
sub title_name
{
	my $self = shift;
	my $title = $self->SUPER::title_name();
	return $title if ($title);
	my $cid = $self->cgi->value('cid');
	$title = sb->string(' &gt; Category');
	if ($cid =~ /^\d+$/ and $self->{'cat'}->{$cid})
	{
		$title .= sb->string(' &gt; ') . $self->{'cat'}->{$cid}->name();
	}
	elsif ($self->cgi->value('__type') eq 'tree')
	{
		$title .= sb->string(' &gt; Tree');
	}
	return $title;
}
# --------------------------------------
# // public functions
sub callback
{
	my $self = shift;
	$self->{'cat'} = { sb::Data->load_as_hash('Category') };
	$self->{'cnt'} = {};
	return ($self->regi)
		? $self->_update(@_)
		: $self->_display(@_);
}
# --------------------------------------
# // private functions
sub _update
{
	my $self = shift;
	my $msg  = undef;
	my $main = ( $self->cgi->value('cid') =~ /^\d+$/ ) 
		? sb::Data->load('Category','id'=>$self->cgi->value('cid')) 
		: undef;
	if ($self->regi() eq 'add')
	{ # --- add new category
		$main = undef; # we can ignore cid in this case.
		if ($self->cgi->value('cat_name') ne '')
		{ # category name is required
			$main = $self->create_category(
				'name' => sb::Text->new($self->cgi->value('cat_name'))->entitized(),
				'main' => $self->cgi->value('cat_main'),
				'sub'  => ($self->cgi->value('cat_main') ne '') ? 1 : undef,
				'text' => $self->cgi->value('cat_text'),
			);
			$self->{'cat'}->{$main->id} = $main if ($main);
		}
		$msg = ($main)
			? sb->string('Your request has been processed.')
			: sb->string('Failed to create a category.');
	}
	elsif ($self->regi() eq 'update')
	{ # --- edit existed category
		($msg,$main) = $self->_update_edit_category($main);
	}
	elsif ( $main )
	{ # modify details
		($msg,$main) = $self->_update_details($main);
	}
	$self->build_list('category_list');
	return $self->_display(
		'message' => $msg,
		'id'      => ($main) ? $main->id : undef
	);
}
sub _display
{
	my $self = shift;
	my %param = (
		'message' => undef,
		'id'      => undef,
		'setup'   => $self->setup_list(),
		@_
	);
	$self->{'cnt'} = $self->_count_entry();
	my $temp = sb::Template->new($self->load_template('file'=>TEMPLATE));
	if ($self->cgi->value('__type') ne 'tree')
	{ # edit mode
		$param{'id'} = $self->_check_id($param{'id'});
		my @cats = $self->_get_categories($param{'id'});
		$temp->num(0);
		$self->listmain(
			'template' => $temp,
			'block'    => 'sb_category_list',
			'objects'  => \@cats,
			'tags'     => {
				'sb_catlist_id'   => 'id',
				'sb_catlist_name' => \&_display_name,
				'sb_catlist_num'  => sub { int((shift)->{'cnt'}->{(shift)->id}) },
				'sb_catlist_sub'  => \&_display_subcat_num,
				'sb_catlist_del'  => \&_display_checkbox,
			},
		);
		for (my $i=0;$i<@cats;$i++)
		{ # for chaging order
			my $lcol = sprintf(ORDER_LEFT ,$cats[$i]->id);
			my $rcol = sprintf(ORDER_RIGHT,$cats[$i]->id);
			$lcol = DEFAULT_COLUMN if ($i == 0);
			$rcol = DEFAULT_COLUMN if ($i == $#cats);
			$temp->num($i);
			$temp->tag('sb_catlist_order'=>sprintf(ORDER_COLUMN,$lcol,$rcol));
		}
		if ($param{'id'} ne '')
		{
			$self->_display_details($temp,$param{'id'})
		}
		else
		{
			$temp->num(0);
			$temp->tag('sb_cat_selector'=>
				$self->category_selector('cat' => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ])
			);
		}
		$temp->num(0);
		$temp->tag('sb_submenu_category_basic'=>' class="current"');
		$temp->block('sb_category_select'  => ($param{'id'} eq '') ? 1 : 0);
		$temp->block('sb_category_one'     => ($param{'id'} eq '') ? 0 : 1);
		$temp->block('sb_category_details' => ($param{'id'} eq '') ? 0 : 1);
		$temp->block('sb_category_basic'   => 1);
		$temp->block('sb_category_tree'    => 0);
	}
	else
	{ # tree mode
		$temp->num(0);
		$temp->tag('sb_category_tree'=>
			$self->_category_tree(
				'cat'  => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
				'num'  => $self->{'cnt'},
				'path' => $self->script_path . '?__mode=category&amp;cid=',
			)
		);
		$temp->tag('sb_submenu_category_tree'=>' class="current"');
		$temp->block('sb_category_tree'  =>1);
	}
	$self->common_parts($temp);
	$self->set_message($temp,$param{'message'});
	return $self->set_main($temp->output)
}
# --------------------------------------
# // private functions - update
sub _update_edit_category
{
	my $self = shift;
	my $main = shift;
	my $msg = '';
	my @cats = sb::Data->load('Category', # load all listed categories
		'cond'  => {'id'=> [split("\0",$self->cgi->value('sel_id'))] },
		'order' => 1,
		'sort'  => 'order'
	);
	if ( $self->cgi->value('submit') ne '' )
	{ # delete selected categories
		my @selected = split("\0",$self->cgi->value('sel'));
		my @dels = ();
		foreach my $cat (@cats)
		{
			my $id = $cat->id;
			if ( grep(/^\Q$id\E$/,@selected) )
			{
				push(@dels,$cat);
				push(@dels,$self->_get_subs($cat)) if ($cat->sub ne '');
			}
		}
		if (@dels)
		{
			map { $_->erase } @dels;
			sb::Data->update(@dels);
		}
		$msg = sprintf(sb->string('Deleted %d item(s).<br />'),$#dels + 1);
	}
	else
	{ # rename or change the order
		my $order = undef;
		foreach my $cat (@cats)
		{
			if ($self->cgi->value('update') ne '')
			{
				my $name = $self->cgi->value('cat_name' . $cat->id);
				$cat->name( sb::Text->new($name)->entitized() ) if ($name ne '');
			}
			next if ($order);
			$order = $cat if ($self->cgi->value('dn' . $cat->id) ne '');
			$order = $cat if ($self->cgi->value('up' . $cat->id) ne '');
		}
		if ($order)
		{
			@cats = $self->change_order(
				'data'      => \@cats,
				'target'    => $order,
				'direction' => ($self->cgi->value('up' . $order->id) ne '') ? +1 : -1,
			);
		}
		sb::Data->update(@cats) if (@cats);
		$msg = sb->string('Modified.<br />');
	}
	if ($main and $msg ne '')
	{ # modify sub-categories
		my @subs = ();
		foreach my $cat (sort { $b->order <=> $a->order } @cats)
		{
			next if ($cat->erased);
			push(@subs,$_->id);
		}
		$main->sub(join(',',@subs) . ',');
		sb::Data->update($main);
	}
	$self->{'cat'} = { sb::Data->load_as_hash('Category') } if ($msg ne '');
	return( $msg, $main );
}
sub _update_details
{
	my $self = shift;
	my $main = shift;
	return sb->string('No such category.') if (!$main);
	my $msg = '';
	if ( $self->cgi->value('detail') ne '' )
	{ # we have the category to update
		if ($self->cgi->value('cat_name') ne '')
		{ # category name is required
			$main->name(sb::Text->new($self->cgi->value('cat_name'))->entitized());
		}
		$main->text($self->cgi->value('cat_text'));
		$main->url($self->cgi->value('cat_tb'));
		$main->temp($self->cgi->value('cat_temp'));
		$main->idx($self->cgi->value('cat_index'));
		$main->dir($self->cgi->value('cat_dir')) if ($self->cgi->value('cat_dir') ne '');
		my @disps = ();
		foreach my $key ('top','list','line','sum')
		{
			my $disp = $self->cgi->value('cat_disp' . $key);
			$disp = ($disp ne 'on') if ($key eq 'line');
			push(@disps,int($disp));
		}
		$main->disp(join(':',@disps) . ':');
		if ($main->idx)
		{ # check the duplication of directory to save.
			my $duplicated = '';
			foreach my $check ( values(%{$self->{'cat'}}) )
			{
				next if ($check->id == $main->id);
				next if (!$check->idx);
				next if ($check->dir ne $main->dir);
				$duplicated = $check->fullname($self->{'cat'});
				last;
			}
			if ($duplicated)
			{
				$msg .= sprintf(sb->string('Directory [%s] to save has been duplicated with other category.<br />',$duplicated));
			}
		}
		if ($self->cgi->value('cat_main') ne $main->main)
		{ # update parent
			my $flag = undef;
			my $old_parent = ($main->main == -1) ? '' : $main->main;
			my $new_parent = ($self->cgi->value('cat_main') == -1) ? '' : $self->cgi->value('cat_main');
			my $old = ($old_parent ne '') ? $self->{'cat'}->{$old_parent} : undef;
			my $new = ($new_parent ne '') ? $self->{'cat'}->{$new_parent} : undef;
			foreach my $check ( values(%{$self->{'cat'}}) )
			{ # checking name whether the name is duplicated or not
				next if ($check->name ne $main->name);
				my $check_parent = ($check->main == -1) ? '' : $check->main;
				$flag = 1 if ($new and $check->main eq $new_parent); # the same sub-category
				$flag = 1 if (!$new and $check_parent eq ''); # the same parent-category
				last if ($flag);
			}
			if (!$flag)
			{ # not duplicated
				$main->main(($new_parent ne '') ? $new_parent : -1);
				if ($new)
				{ # needs update of parent category as well
					$new->add_sub($main->id);
					sb::Data->update($new);
				}
				if ($old)
				{
					$old->remove_sub($main->id);
					sb::Data->update($old);
				}
			}
			else
			{
				$msg .= sb->string('The category already exists.<br />');
			}
		}
	} # end of if ( $self->cgi->value('detail') ne '' )
	elsif ( $self->cgi->value('del') ne '' )
	{ # delete indicated category
		my $parent = ($main->main > -1 and $main->main ne '') ? $self->{'cat'}->{$main->main} : undef;
		my @dels = ( $main );
		push(@dels,$self->_get_subs($main)) if ($main->sub ne '');
		map { $_->erase } @dels;
		if ($parent)
		{
			$parent->remove_sub($main->id);
			sb::Data->update($parent);
		}
		sb::Data->update(@dels);
		$main = undef;
		$msg = sprintf(sb->string('Deleted %d item(s).<br />'),$#dels + 1);
	}
	if ($main)
	{
		sb::Data->update($main);
		$msg .= sb->string('Modified.<br />');
	}
	$self->{'cat'} = { sb::Data->load_as_hash('Category') } if ($msg ne '');
	return( $msg, $main );
}
# --------------------------------------
# // private functions - display
sub _display_details
{
	my $self = shift;
	my $temp = shift;
	my $id = shift;
	my $editor = $self->get_editor()->new('user' => $self->user);
	my $main = $self->{'cat'}->{$id};
	return if (!$main);
	$temp->num(0);
	$temp->tag('sb_category_editor' => $editor->display_editor(
		'id' => 'cat_text',
		'content' => sb::Text->new($main->text)->entitized(),
	));
	$temp->tag('sb_cat_id'=>$main->id);
	$temp->tag('sb_cat_name'=>$main->name);
	$temp->tag('sb_cat_tb'=>$main->url);
	foreach my $key ('top','list','sum')
	{
		$self->select_option(
			'cms'      => $temp,
			'tag'      => 'sb_cat_disp' . $key . '_',
			'selected' => $main->get_option($key),
		);
	}
	$temp->tag('sb_cat_displine'=>'checked="checked"') if (!$main->get_option('line'));
	$self->template_selector(
		'cms' => $temp,
		'tag' => 'sb_cat_temp',
		'now' => ($main->temp == -1) ? undef : $main->temp,
	);
	# if ($self->conf->value('conf_entry_archive') ne 'None')
	{ # settings for stored directory and category index
		my @dirs = sort { $a cmp $b } sb::Utils->writable_dir($self->conf->value('conf_dir_log'));
		my $selector = '';
		foreach my $dir (@dirs)
		{ # directory setting
			$selector .= '<option value="' . $dir . '"';
			$selector .= ' selected="selected"' if ($main->dir eq $dir);
			$selector .= '>' . $dir . '</option>';
		}
		$temp->num(0);
		$temp->tag('sb_cat_dir'=>$selector);
		$temp->tag('sb_cat_index_' . $main->idx=>'selected="selected"') if ($main->idx);
		$temp->block('sb_category_dir'=>1);
		$temp->block('sb_category_index'=>1);
	} # end of settings for stored directory and category index
	my $navi = $self->_category_navigator(
		'id'  => $main->id,
		'tag' => '<a href="' . $self->script_path . '?__mode=category&amp;cid=%d">%s</a>',
		'now' => $main->id,
	);
	$navi .= ' (<a href="' . $self->script_path . '?__mode=list&amp;dispcat=';
	$navi .= $main->id . '">' . int($self->{'cnt'}->{$main->id}) . '</a>)';
	$temp->num(0);
	$temp->tag('sb_cat_navi'=>$navi);
	$temp->tag('sb_cat_selector'=>
		$self->category_selector(
			'cat'    => [ sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) ],
			'now'    => $main->main,
			'except' => $main->id,
		)
	);
}
sub _check_id
{
	my $self = shift;
	my $preferred = shift;
	$preferred = $self->cgi->value('cid') if ($self->cgi->value('cid') =~ /^\d+$/);
	return $preferred if ($self->{'cat'}->{$preferred});
	return undef;
}
sub _get_categories
{
	my $self = shift;
	my $id = shift;
	my @cats = ();
	if ($id ne '' and $self->{'cat'}->{$id})
	{
		foreach my $cat ( split(',',$self->{'cat'}->{$id}->sub) )
		{
			next if ($cat eq '');
			next if (!$self->{'cat'}->{$cat});
			push(@cats,$self->{'cat'}->{$cat});
		}
		return @cats;
	}
	foreach my $cat ( sort { $b->order <=> $a->order } values(%{$self->{'cat'}}) )
	{
		my $parent = ($cat->main == -1) ? '' : $cat->main;
		next if ($parent ne '');
		push(@cats,$cat);
	}
	return @cats;
}
sub _get_subs
{ # get sub-categories recursively
	my $self = shift;
	my $cat  = shift;
	my @subs = split(',',$cat->sub);
	my @objs = ();
	foreach (@subs)
	{
		my $obj = $self->{'cat'}->{$_};
		if ($obj)
		{
			push(@objs,$obj) ;
			push(@objs,$self->_get_subs($obj)) if ($obj->sub ne '');
		}
	}
	return @objs;
}
sub _category_tree
{ # category tree
	my $self  = shift;
	my %param = (
		'cat'    => [],
		'branch' => undef,   
		'num'    => {},
		'path'   => undef,
		'class'  => -1,
		@_
	);
	my $list = '';
	my $num = $param{'class'};
	foreach my $cat ( @{$param{'cat'}} )
	{
		my $parent = ($cat->main == -1) ? '' : $cat->main;
		next if (!defined($param{'branch'}) and $parent ne '');
		next if ( defined($param{'branch'}) and $parent ne $param{'branch'});
		my $dir = ($self->conf->value('conf_entry_archive') ne 'Individual') 
			? undef
			: ($cat->dir ne '') ? $cat->dir : $self->conf->value('conf_dir_log');
		$num++;
		$list .= ($num % 2) ? '<li class="odd">' : '<li class="even">';
		$list .= '<span class="dir">' . $dir . '</span>' if ($dir);
		$list .= '<a href="' . $param{'path'} . $cat->id . '">' . $cat->name . '</a>';
		$list .= ' (' . int($param{'num'}->{$cat->id}) . ')';
		if ($cat->sub ne '')
		{
			$list .= "\n" . '<ul>';
			$list .= $self->_category_tree(
				'cat'    => $param{'cat'},
				'branch' => $cat->id,
				'num'    => $param{'num'},
				'path'   => $param{'path'},
				'class'  => $num,
			);
			$list .= '</ul>';
		}
		$list .= '</li>' . "\n";
	}
	return $list;
}
sub _category_navigator
{
	my $self = shift;
	my %param = (
		'id'  => undef,
		'tag' => '',
		'now' => undef,
		@_
	);
	return( undef ) if (!defined($param{'id'}));
	return( undef ) if (!$self->{'cat'}->{$param{'id'}});
	if ( $self->{'cat'}->{$param{'id'}}->main eq '')
	{
		return ($param{'now'} eq $param{'id'})
			? $self->{'cat'}->{$param{'id'}}->name
			: sprintf($param{'tag'},$param{'id'},$self->{'cat'}->{$param{'id'}}->name);
	}
	else
	{
		my $navi = $self->_category_navigator(
			'id'  => $self->{'cat'}->{$param{'id'}}->main,
			'tag' => $param{'tag'},
			'now' => $param{'now'},
		);
		$navi .= NAVI_ARROW if ($navi);
		return ($param{'now'} eq $param{'id'})
			? $navi . $self->{'cat'}->{$param{'id'}}->name
			: $navi . sprintf($param{'tag'},$param{'id'},$self->{'cat'}->{$param{'id'}}->name);
	}
}
# --------------------------------------
# // private functions - list handlers
sub _display_name
{
	my $self = shift;
	my $obj  = shift;
	return '<input type="text" name="cat_name' . $obj->id . '" value="' . $obj->name . '" size="20" class="messagetext" />';
}
sub _display_subcat_num
{
	my $self = shift;
	my $obj  = shift;
	my $sub  = $obj->sub;
	return int( $sub =~ s/,/,/g );
}
sub _display_checkbox
{
	my $self = shift;
	my $obj  = shift;
	return '<input type="checkbox" name="sel" value="' . $obj->id . '" class="form-control" />';
}
sub _count_entry
{
	my $self = shift;
	my %count = ();
	my @entries = sb::Data->load('Entry','limit'=>'add');
	foreach my $entry ( @entries )
	{
		next if ($entry->cat eq '' and $entry->add eq '');
		my %internal_count;
		my @cats = ($entry->cat);
		push(@cats,split(',',$entry->add));
		foreach my $check ( @cats )
		{
			next if ($check eq '');
			next if ($internal_count{$check}++);
			next if (!$self->{'cat'}->{$check});
			$count{$check}++;
		}
	}
	return \%count;
}
1;
__END__
