# sb::Language::ja
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package sb::Language::ja;
use strict;
use sb::Language ();
use base qw( sb::Language );
use vars qw( $VERSION );
$VERSION = '0.07';
# 0.07 [2008/11/24] modified setup to output debug log
# 0.06 [2008/10/30] added mailcode
# 0.05 [2007/12/07] fixed a bug in convert handling $in_code incorrectly
# 0.04 [2007/08/07] added $aConvertSpecialChar to convert special characters for utf8
# 0.03 [2007/03/08] changed setup to add 'Showa Day'
# 0.02 [2006/03/17] changed bracket rule
# 0.01 [2005/10/25] refactored
# 0.00 [2005/01/17] generated
# --------------------------------------
# // defines / static variables
my $aConvertSpecialChar = 1;
# --------------------------------------
# // override base class
sub setup
{
	my $self = shift;
	$self->SUPER::setup(@_);
	eval { require "Jcode.pm" }; # load Jcode.pm
	if ( $@ )
	{
		sb->debug("fail to load Jcode.pm");
		return;
	}
	my $check = ord("漢"); # 「漢」と書いて「おとこ」と読む
	if ($check == 0xb4 or $check ==  -76)
	{
		$self->charset('EUC-JP');
		$self->charcode('euc');
	}
	elsif ($check == 0x8a or $check == -118)
	{
		$self->charset('Shift_JIS');
		$self->charcode('sjis');
	}
	elsif ($check == 0xe6 or $check ==  -26)
	{
		$self->charset('UTF-8');
		$self->charcode('utf8');
	}
	elsif ($check == 0x1b)
	{
		$self->charset('iso-2022-jp');
		$self->charcode('jis');
	}
	else
	{
		die("unknown charcode");
	}
	$self->string('week_ja'     =>['日','月','火','水','木','金','土']);
	$self->string('week_jalong' =>['日曜日','月曜日','火曜日','水曜日','木曜日','金曜日','土曜日']);
	$self->string('month_ja'    =>['一月','二月','三月','四月','五月','六月','七月','八月','九月','十月','十一月','十二月']);
	$self->string('month_jalong'=>['睦月','如月','弥生','卯月','皐月','水無月','文月','葉月','長月','神無月','霜月','師走']);
	# holidays
	$self->string('New Year\'s Day'          =>'元日');
	$self->string('Foundation Day'           =>'建国記念日');
	$self->string('Showa Day'                =>'昭和の日');
	$self->string('Green Day'                =>'みどりの日');
	$self->string('Constitution Memorial Day'=>'憲法記念日');
	$self->string('Children\'s Day'          =>'こどもの日');
	$self->string('Culture Day'              =>'文化の日');
	$self->string('Labor Thanksgiving Day'   =>'勤労感謝の日');
	$self->string('Emperor\'s Birthday'      =>'天皇誕生日');
	$self->string('Adult\'s Day'             =>'成人の日');
	$self->string('Sports Day'               =>'体育の日');
	$self->string('Marine Day'               =>'海の日');
	$self->string('Old People\'s Day'        =>'敬老の日');
	$self->string('Vernal Equinox'           =>'春分の日');
	$self->string('Autumnal Equinoxy'        =>'秋分の日');
	$self->string('National Hoilday'         =>'国民の休日');
	$self->string('A holiday in lieu'        =>'(振替休日)');
	return $self;
}
sub convert
{
	my $self = shift;
	my ($text,$outcode,$in_code) = @_;
	$in_code = $self->checkcode($text) if ($in_code eq '');
	return $text if ($outcode eq $in_code);
	&Jcode::convert(\$text,$outcode,$in_code);
	if ($aConvertSpecialChar and $outcode eq 'utf8')
	{
		$text =~ s/\xC2\xA5/\\/g; # yen sign vs back slash
		$text =~ s/\xE2\x80\xBE/\x7E/g; # macron vs tilde
	}
	return $text;
}
sub checkcode
{
	my $self = shift;
	my $text = shift;
	return &Jcode::getcode($text);
}
sub mailcode
{
	my $self = shift;
	return 'jis';
}
# --------------------------------------
# // public function
sub enable_convert_option
{
	my $class = shift;
	$aConvertSpecialChar = shift;
	return;
}
1;
__END__
